@@ -21,17 +21,30 @@
 ##' @param df.fixed [for tCopula():] logical indicating if 'df' is fixed or a parameter
 ##' @return integer
 ##' @author Martin Maechler
-## NOTE: This is related to  validRho() in ./Classes.R
+## NOTE: This is related to  validRho() (== validityMethod for "ellipCopula") in ./Classes.R
 npar.ellip <- function(dim, dispstr, df.fixed = TRUE) {
     (!df.fixed) + # add 1 if 'df' is not fixed
     switch(dispstr, ## also checking for correct 'dispstr'
-	   "ar1" =, "ex" = 1 ,
+	   "ar1" =,
+           "ex" = 1 ,
 	   "un" = dim * (dim - 1) / 2,
 	   "toep" = dim - 1,
 	   ## otherwise
 	   return("'dispstr' not supported (yet)"))
 }
 
+## Lower bound for the rho_j
+lowbnd.rho.ellip <- function(dim, dispstr, pdim = npar.ellip(dim, dispstr)) {
+    switch(dispstr, ## also checking for correct 'dispstr'
+	   "ex" = rep( - 1 / (dim-1), pdim),
+	   "ar1"= rep( -1, pdim),
+	   ## These bounds are not at all tight, but not simple box-constraints anyway:
+	   "un" = ,
+	   "toep" = rep(-1, pdim),# -> ~/R/MM/NUMERICS/toeplitz-posdef.R for some bounds
+	   ## otherwise
+	   return("'dispstr' not supported (yet)"))
+}
+
 ellipCopula <- function(family = c("normal", "t"), param = NA_real_, dim = 2L,
                         dispstr = "ex", df = 4, ...)
 {
@@ -40,6 +53,23 @@ ellipCopula <- function(family = c("normal", "t"), param = NA_real_, dim = 2L,
          "t" =           tCopula(param, dim = dim, dispstr = dispstr, df = df, ...))
 }
 
+setMethod(describeCop, c("ellipCopula", "character"), function(x, kind, prefix="", ...) {
+    kind <- match.arg(kind)
+    cl <- sub("Copula$", "", class(x)) # -> "t" / "normal"
+    if (cl == "normal") cl <- "Normal"
+    clNam <- paste0(prefix, cl, if(nchar(cl) <= 2) "-" else " ", "copula")
+    if(kind == "very short") # e.g. for show() which has more parts
+        return(clNam)
+    ## else
+    d <- dim(x)
+    ch <- paste(paste0(clNam, ", dim. d ="), d)
+    switch(kind <- match.arg(kind),
+	   short = ch,
+	   long = paste0(ch, "\n", prefix, "param.: ", dputNamed(getTheta(x, named=TRUE))),
+	   stop("invalid 'kind': ", kind))
+})
+
+
 iTauEllipCopula <- function(copula, tau) sinpi(tau / 2)
 
 ## iRho --> only for normalCopula
