@@ -69,9 +69,9 @@ gumbelCopula <- function(param = NA_real_, dim = 2L,
   }
 
   ## get expressions of cdf and pdf
-  cdfExpr <- function(n) {
+  cdfExpr <- function(d) {
     expr <- "( - log(u1))^alpha"
-    for (i in 2:n) {
+    for (i in 2:d) {
       cur <- paste0( "(-log(u", i, "))^alpha")
       expr <- paste(expr, cur, sep=" + ")
     }
@@ -79,24 +79,24 @@ gumbelCopula <- function(param = NA_real_, dim = 2L,
     parse(text = expr)
   }
 
-  pdfExpr <- function(cdf, n) {
+  pdfExpr <- function(cdf, d) {
     val <- cdf
-    for (i in 1:n) {
+    for (i in 1:d) {
       val <- D(val, paste0("u", i))
     }
     val
   }
 
   cdf <- cdfExpr((dim <- as.integer(dim)))
-  pdf <- if (dim <= 6) pdfExpr(cdf, dim) else NULL
+  pdf <- if (dim <= 6) pdfExpr(cdf, dim) # else NULL
   new("gumbelCopula",
-             dimension = dim,
-             parameters = param,
-             exprdist = c(cdf = cdf, pdf = pdf),
-             param.names = "param",
-             param.lowbnd = 1,
-             param.upbnd = Inf,
-             fullname = "Gumbel copula family; Archimedean copula; Extreme value copula")
+      dimension = dim,
+      parameters = param,
+      exprdist = c(cdf = cdf, pdf = pdf),
+      param.names = "param",
+      param.lowbnd = 1,
+      param.upbnd = Inf,
+      fullname = "<deprecated slot>")# "Gumbel copula family; Archimedean copula; Extreme value copula"
 }
 
 
@@ -124,7 +124,7 @@ pgumbelCopula <- function(copula, u) {
   if(!is.matrix(u)) u <- matrix(u, ncol = dim)
   for (i in 1:dim) assign(paste0("u", i), u[,i])
   alpha <- copula@parameters[1] # needed in eval():
-  pmax(0, eval(copula@exprdist$cdf))
+  pmax(eval(copula@exprdist$cdf), 0)
 }
 
 dgumbelCopula <- function(u, copula, log=FALSE, ...) {
@@ -148,31 +148,20 @@ dgumbelCopula.pdf <- function(u, copula, log=FALSE) {
   else  c(eval(gumbelCopula.pdf.algr[dim]))
 }
 
-tauGumbelCopula <- function(copula) {
-  alpha <- copula@parameters[1]
-  1 - 1/alpha
-}
-
-lambdaGumbelCopula <- function(copula, ...) {
-  alpha <- copula@parameters
-  upper <- 2 - 2^(1/alpha)
-  c(lower=0, upper=upper)
-}
-
 
 iTauGumbelCopula <- function(copula, tau) {
-  if (any(tau < 0)) warning("some tau < 0")
-  ifelse(tau <= 0, 1, 1/(1 - tau))
+  r <- 1/(1 - tau)
+  if (any(neg <- tau < 0)) {
+    warning("some tau < 0")
+    r[neg] <- 1
+  }
+  r
 }
 
 
 gumbelRhoFun <- function(alpha) {
-  ss <- .gumbelRho$ss
-  forwardTransf <- .gumbelRho$trFuns$forwardTransf
-  valFun <- .gumbelRho$assoMeasFun$valFun
-  theta <- forwardTransf(alpha, ss)
-
-  c(valFun(theta))
+  theta <- .gumbelRho$trFuns$forwardTransf(alpha, .gumbelRho$ss)
+  c(.gumbelRho$assoMeasFun$valFun(theta))
 }
 
 gumbeldRho <- function(alpha) {
@@ -185,49 +174,26 @@ gumbeldRho <- function(alpha) {
   c(valFun(theta, 1)) * forwardDer(alpha, ss)
 }
 
-rhoGumbelCopula <- function(copula) {
-  alpha <- copula@parameters[1]
-  gumbelRhoFun(alpha)
-}
-
 
 iRhoGumbelCopula <- function(copula, rho) {
   if (any(rho < 0)) warning("rho is out of the range [0, 1]")
   gumbelRhoInv <- approxfun(x = .gumbelRho$assoMeasFun$fm$ysmth,
                             y = .gumbelRho$assoMeasFun$fm$x, rule = 2)
-
   ss <- .gumbelRho$ss
   theta <- gumbelRhoInv(rho)
   .gumbelRho$trFuns$backwardTransf(theta, ss)
 }
 
 
-dTauGumbelCopula <- function(copula) {
-  return( 1 / copula@parameters^2 )
-}
-
-dRhoGumbelCopula <- function(copula) {
-  alpha <- copula@parameters[1]
-  gumbeldRho(alpha)
-}
-
 setMethod("rCopula", signature("numeric", "gumbelCopula"), rgumbelCopula)
 
 setMethod("pCopula", signature("matrix", "gumbelCopula"),
 	  ## was  pgumbelCopula
 	  function(u, copula, ...) .pacopula(u, copGumbel, theta=copula@parameters))
-setMethod("pCopula", signature("numeric", "gumbelCopula"),
-	  ## was  pgumbelCopula
-	  function(u, copula, ...) pacopula(u, copGumbel, theta=copula@parameters))
-
 setMethod("dCopula", signature("matrix", "gumbelCopula"),
 	  ## was  dgumbelCopula.pdf
 	  function (u, copula, log = FALSE, checkPar=TRUE, ...)
 	  copGumbel@dacopula(u, theta=copula@parameters, log=log, checkPar=checkPar, ...))
-setMethod("dCopula", signature("numeric", "gumbelCopula"),
-	  function (u, copula, log = FALSE, checkPar=TRUE, ...)
-	  copGumbel@dacopula(rbind(u, deparse.level=0L), theta=copula@parameters,
-			     log=log, ...))
 
 
 setMethod("A", signature("gumbelCopula"), AGumbel)
@@ -246,13 +212,18 @@ setMethod("diPsi", signature("gumbelCopula"),
       })
 
 
+setMethod("tau", "gumbelCopula", function(copula) 1 - 1/copula@parameters)
+
+setMethod("rho", "gumbelCopula", function(copula) gumbelRhoFun(copula@parameters))
+
+setMethod("lambda", signature("gumbelCopula"), function(copula, ...)
+  c(lower = 0, upper = 2 - 2^(1/copula@parameters)))
 
-setMethod("tau", signature("gumbelCopula"), tauGumbelCopula)
-setMethod("rho", signature("gumbelCopula"), rhoGumbelCopula)
-setMethod("lambda", signature("gumbelCopula"), lambdaGumbelCopula)
 
 setMethod("iTau", signature("gumbelCopula"), iTauGumbelCopula)
 setMethod("iRho", signature("gumbelCopula"), iRhoGumbelCopula)
 
-setMethod("dRho", signature("gumbelCopula"), dRhoGumbelCopula)
-setMethod("dTau", signature("gumbelCopula"), dTauGumbelCopula)
+
+setMethod("dRho", "gumbelCopula", function(copula) gumbeldRho(copula@parameters))
+
+setMethod("dTau", "gumbelCopula", function(copula) 1 / copula@parameters^2)
