@@ -32,7 +32,7 @@ A..Z <- function(x, alpha, I.alpha = 1 - alpha)
 ##' @param x numeric vector
 ##' @return numeric vector of values tan(pi*x)
 ##' @author Martin Maechler
-tanpi <- function(x) tan(pi * (x %% 1))
+##' tanpi <- function(x) tan(pi * (x %% 1))
 
 ##' @title cos(pi/2 * x), exact for integer x
 ##' @param x numeric vector
@@ -47,7 +47,7 @@ cospi2 <- function(x) {
 	r[i[x[i] == 2]] <- -1
     }
     io <- which(!isI)
-    r[io] <- cos(pi/2 * x[io])
+    r[io] <- cospi(x[io]/2)
     r
 }
 
@@ -65,7 +65,7 @@ cospi2 <- function(x) {
 ##' @author Martin Maechler, based on Diethelm Wuertz's code in fBasics
 rstable1R <- function(n, alpha, beta, gamma = 1, delta = 0, pm = 1)
 {
-    stopifnot((la <- length(alpha)) >= 1, (lb <- length(beta)) >= 1,
+    stopifnot((la <- length(alpha)) >= 1, length(beta) >= 1,
 	      length(gamma) >= 1, length(delta) >= 1,
 	      0 < alpha, alpha <= 2, abs(beta) <= 1,
 	      length(pm) == 1, pm %in% 0:1)
@@ -157,7 +157,7 @@ rstable1R <- function(n, alpha, beta, gamma = 1, delta = 0, pm = 1)
 ##' @author Martin Maechler
 rstable1C <- function(n, alpha, beta, gamma = 1, delta = 0, pm = 1)
 {
-    stopifnot((la <- length(alpha)) >= 1, (lb <- length(beta)) >= 1,
+    stopifnot(length(alpha) >= 1, length(beta) >= 1,
 	      length(gamma) >= 1, length(delta) >= 1,
 	      0 < alpha, alpha <= 2, abs(beta) <= 1, gamma >= 0,
               length(pm) == 1, pm %in% 0:1)
